/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.    
 */
package org.apache.tuscany.sca.assembly;

/**
 * Represent a binding that supports SCA wiring between component references and services
 * 
 * @version $Rev: 567203 $ $Date: 2007-08-18 03:08:36 +0100 (Sat, 18 Aug 2007) $
 */
public interface WireableBinding extends Binding, Cloneable {

    /**
     * @param component
     */
    void setTargetComponent(Component component);
    
    /**
     * @param service
     */
    void setTargetComponentService(ComponentService service);
    
    /**
     * @param binding
     */
    void setTargetBinding(Binding binding);
    
    /**
     * If a reference targets in a component running in a separate
     * node then its binding will be set remote until runtime
     * 
     * @param isRemote
     */
    void setRemote(boolean isRemote);
    
    /**
     * @return the flag indicating whether the binding targets a remote component
     */
    boolean isRemote();
     
    /**
     * @return
     */
    Binding getTargetBinding();
    
    /**
     * @return
     */
    Component getTargetComponent();
    
    /**
     * @return
     */
    ComponentService getTargetComponentService();

    /**
     * Clone the binding
     * @return
     */
    Object clone() throws CloneNotSupportedException;

}
