/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.    
 */
package org.apache.tuscany.sca.core.invocation;

import org.apache.tuscany.sca.core.factory.ObjectCreationException;
import org.apache.tuscany.sca.core.factory.ObjectFactory;
import org.apache.tuscany.sca.runtime.RuntimeComponent;
import org.apache.tuscany.sca.runtime.RuntimeComponentReference;
import org.osoa.sca.CallableReference;

/**
 * Uses a wire to return a callable reference
 * 
 * @version $Rev: 568331 $ $Date: 2007-08-22 00:32:51 +0100 (Wed, 22 Aug 2007) $
 */
public class CallableReferenceObjectFactory implements ObjectFactory<CallableReference<?>> {
    private CallableReference<?> callableReference;

    /**
     * Constructor.
     * 
     * To support the @Reference protected CallableReference<MyService> ref;
     * 
     * @param interfaze the interface to inject on the client
     * @param wire the backing wire
     * @param proxyService the wire service to create the proxy
     * @throws NoMethodForOperationException
     */
    public CallableReferenceObjectFactory(Class<?> businessInterface,
                                          RuntimeComponent component,
                                          RuntimeComponentReference reference) {
        this.callableReference = component.getComponentContext().getServiceReference(businessInterface, reference);
    }

    public CallableReference<?> getInstance() throws ObjectCreationException {
        return callableReference;
    }

}
