/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.tuscany.sca.databinding.saxon;

import net.sf.saxon.value.ObjectValue;
import net.sf.saxon.value.Value;

import org.apache.tuscany.sca.databinding.PullTransformer;
import org.apache.tuscany.sca.databinding.TransformationContext;
import org.apache.tuscany.sca.databinding.impl.BaseTransformer;

/**
 * Transforms generic java objects to Value objects needed by Saxon parser
 * @version $Rev: 566518 $ $Date: 2007-08-16 08:21:34 +0100 (Thu, 16 Aug 2007) $
 */
public class Object2ValueTransformer extends BaseTransformer<Object, Value>
		implements PullTransformer<Object, Value> {
	
	@Override
	protected Class getSourceType() {
		return Object.class;
	}

	@Override
	protected Class getTargetType() {
		return Value.class;
	}

	@Override
	public int getWeight() {
		return 10000;
	}

	public Value transform(Object source, TransformationContext context) {
		return new ObjectValue(source);
	}

}
